class Options(dict):
    """\
    Options dictionary. An instance of this is attached to each
    :class:`fresco.core.FrescoApp` instance, as a central store for
    configuration options.
    """

    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, key, value):
        self[key] = value

    def update_from_file(self, path):
        """
        Update the instance with any symbols found in the python source file at
        `path`.
        """
        with open(path, 'r') as f:
            source = f.read()
            config_code = compile(source, f.name, 'exec')
        ns = {'__file__': path, 'options': self}
        exec config_code in ns
        del ns['__file__']
        del ns['options']
        self.update(ns)

    def update_from_object(self, ob):
        """
        Update the instance with any symbols listed in object `ob`
        """
        self.update(dict((k, getattr(ob, k)) for k in dir(ob)))



from flea import TestAgent
from nose.tools import assert_equal, assert_raises

from fresco import FrescoApp, GET, Response
from fresco.decorators import extract_getargs
from fresco.exceptions import BadRequest

class TestExtractGetArgs(object):

    def test_mixed_with_dispatcher_args(self):

        app = FrescoApp()

        @app.route(r'/<arg1:unicode>/<arg2:unicode>', GET)
        @extract_getargs(arg1=unicode, arg2=int)
        def handler(request, arg1, arg2):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__, arg2, type(arg2).__name__)
            ])

        assert_equal(
            TestAgent(app).get('/foo/29').body,
            "Received u'foo':unicode, 29:int"
        )

    def test_query_args(self):

        @extract_getargs(arg1=unicode, arg2=int)
        def view(request, arg1, arg2):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__, arg2, type(arg2).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        assert_equal(
            TestAgent(app).get('/?arg1=foo;arg2=29').body,
            "Received u'foo':unicode, 29:int"
        )

    def test_missing_args_with_one_default_argument(self):

        @extract_getargs(arg1=unicode, arg2=int)
        def view(request, arg1, arg2):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__, arg2, type(arg2).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        assert_equal(
            TestAgent(app).get('/?arg1=foo', check_status=False).response.status_code,
            400
        )

    def test_missing_args(self):

        @extract_getargs(arg1=unicode, arg2=int)
        def view(request, arg1, arg2=None):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__, arg2, type(arg2).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        r = TestAgent(app).get('/?arg1=foo')
        assert_equal(r.response.status, '200 OK')
        assert_equal(r.response.body, "Received u'foo':unicode, None:NoneType")

    def test_conversion_error_without_default(self):

        @extract_getargs(arg1=int)
        def view(request, arg1):
            return Response([])

        app = FrescoApp()
        app.route('/', GET, view)
        assert_equal(TestAgent(app).get('/?arg1=foo', check_status=False).response.status_code, 400)

    def test_conversion_error_with_strict_checking(self):

        @extract_getargs(arg1=int, strict_checking=True)
        def view(request, arg1=None):
            return Response([
                'Received %r:%s' % (arg1, type(arg1).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        assert_equal(TestAgent(app).get('/?arg1=foo', check_status=False).response.status_code, 400)

    def test_no_conversion_error_with_default(self):

        @extract_getargs(arg1=int)
        def view(request, arg1=None):
            return Response([
                'Received %r:%s' % (arg1, type(arg1).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        assert_equal(TestAgent(app).get('/?arg1=foo').body, 'Received None:NoneType')




from contextlib import contextmanager

@contextmanager
def atomic_writer(path, mode=0644):
    """
    Write to path in an atomic operation. Auto creates any directories leading up to ``path``
    """
    dirname = os.path.dirname(path)
    makedir(dirname)
    tmpfile = NamedTemporaryFile(delete=False, dir=os.path.dirname(path))
    yield tmpfile
    tmpfile.close()
    os.rename(tmpfile.name, path)

def makedir(path):
    """
    Create a directory at ``path``. Unlike ``os.makedirs`` don't raise an error if ``path`` already exists.
    """
    try:
        os.makedirs(path)
    except OSError, e:
        # Path already exists or cannot be created
        if not os.path.isdir(path):
            raise




"""\
Utility functions for URL building and manipulation
"""

import fresco
from fresco.multidict import MultiDict
from urllib import quote_plus

__all__ = 'join_path', 'strip_trailing_slashes'


def join_path(a, b):
    """
    Join two URL path segments together.
    Unlike ``os.path.join``, this removes leading slashes before joining the
    paths, so absolute paths are appended as if they were relative.

    Synopsis::

        >>> join_path('/', '/abc')
        '/abc'

        >>> join_path('/', '/abc/')
        '/abc/'

        >>> join_path('/foo', '/bar/')
        '/foo/bar/'

        >>> join_path('/foo/', '/bar/')
        '/foo/bar/'

        >>> join_path('/foo', 'bar')
        '/foo/bar'
    """

    if not b:
        return a

    while a and a[-1] == '/':
        a = a[:-1]

    while b and b[0] == '/':
        b = b[1:]

    return a + '/' + b


def strip_trailing_slashes(path):
    """\
    Remove trailing slashes from the given path.

    :param path: a path, eg ``'/foo/bar/'``
    :return: The path with trailing slashes removed, eg ``'/foo/bar'``
    """
    while path and path[-1] == '/':
        path = path[:-1]
    return path


def normpath(path):
    """
    Return ``path`` normalized to remove '../' etc.

    This differs from ``posixpath.normpath`` in that:

    * trailing slashes are preserved
    * multiple consecutive slashes are always condensed to a single slash

    Examples::

        >>> normpath('/hello/../goodbye')
        '/goodbye'
        >>> normpath('//etc/passwd')
        '/etc/passwd'
        >>> normpath('../../../../etc/passwd')
        'etc/passwd'
        >>> normpath('/etc/passwd/')
        '/etc/passwd/'
    """
    segments = path.split('/')
    newpath = []
    last = len(segments) - 1
    for pos, seg in enumerate(segments):
        if seg == '.':
            seg = ''

        if seg == '':
            allow_empty = (
                pos == 0
                or pos == last and newpath and newpath[-1] != ''
                or pos == last and newpath == ['']
            )
            if not allow_empty:
                continue

        if seg == '..':
            if newpath and newpath != ['']:
                newpath.pop()
            continue

        newpath.append(seg)

    return '/'.join(newpath)


def make_query(data=None, separator=';', charset=None, **kwargs):
    """\
    Return a query string formed from the given dictionary data.

    Note that the pairs are separated using a semicolon, in accordance with
    `the W3C recommendation
<http://www.w3.org/TR/1999/REC-html401-19991224/appendix/notes.html#h-B.2.2>`_

    If no encoding is given, unicode values are encoded using the character set
    specified by ``fresco.DEFAULT_CHARSET``.

    Basic usage::

        >>> make_query({'search': u'foo bar', 'type': u'full'})
        'search=foo+bar;type=full'

        >>> make_query(search=u'foo', type=u'quick')
        'search=foo;type=quick'

    Changing the separator to '&'::

        >>> make_query({'search': u'foo', 'type': u'full'}, separator='&')
        'search=foo&type=full'

    Multiple values can be provided per key::

        >>> make_query(search=[u'foo', u'bar'])
        'search=foo;search=bar'

    Exclude values by setting them to ``None``::

        >>> make_query(x=1, y=None)
        'x=1'

    """
    if isinstance(data, MultiDict):
        items = data.allitems()
    elif isinstance(data, dict):
        items = data.items()
    elif data is None:
        items = []
    else:
        items = list(data)
    items += kwargs.items()

    if charset is None:
        charset = fresco.DEFAULT_CHARSET

    # Sort data items for a predictable order in tests
    items.sort()
    items = _repeat_keys(items)
    items = ((k, v) for k, v in items if v is not None)
    return separator.join([
        _qs_frag(k, v, charset=charset) for k, v in items
    ])


def _qs_frag(key, value, charset=None):
    u"""
    Return a fragment of a query string in the format 'key=value'.

    >>> _qs_frag('search-by', 'author, editor')
    'search-by=author%2C+editor'

    If no encoding is specified, unicode values are encoded using the character
    set specified by ``fresco.DEFAULT_CHARSET``.
    """
    if charset is None:
        charset = fresco.DEFAULT_CHARSET

    return quote_plus(_make_bytestr(key, charset)) \
            + '=' \
            + quote_plus(_make_bytestr(value, charset))


def _make_bytestr(ob, charset):
    u"""
    Return a byte string conversion of the given object. If the object is a
    unicode string, encode it with the given encoding.

    Example::

        >>> _make_bytestr(1, 'utf-8')
        '1'
        >>> _make_bytestr(u'a', 'utf-8')
        'a'
    """
    if isinstance(ob, unicode):
        return ob.encode(charset)
    return str(ob)

def _repeat_keys(iterable):
    u"""
    Return a list of ``(key, scalar_value)`` tuples given an iterable
    containing ``(key, iterable_or_scalar_value)``.

    Example::

        >>> list(
        ...     _repeat_keys([('a', 'b')])
        ... )
        [('a', 'b')]
        >>> list(
        ...     _repeat_keys([('a', ['b', 'c'])])
        ... )
        [('a', 'b'), ('a', 'c')]
    """

    for key, value in iterable:
        if isinstance(value, basestring):
            value = [value]
        else:
            try:
                value = iter(value)
            except TypeError:
                value = [value]

        for subvalue in value:
            yield key, subvalue



__all__ = 'ResponseException', 'NotFound', 'NotFoundFinal', 'Unauthorized',\
          'Forbidden', 'BadRequest', 'Redirect', 'RedirectTemporary',\
          'RedirectPermanent'

from fresco.response import Response


class ResponseException(Exception):
    """\
    An exception class with an associated :class:`fresco.response.Response`
    instance that will render a suitable error page.
    """
    response = None


class _NotFound(ResponseException):
    response = Response.not_found()


class NotFound(_NotFound):
    """\
    Raised when a view needs to signal a 404 not found condition. Other routes
    will be given the opportunity to handle the request, if no others are able
    to, 404 not found response will be returned. """


class NotFoundFinal(_NotFound):
    """\
    Return a 404 response directly, even if there may be other views available
    to handle the request.
    """


class BadRequest(ResponseException):
    """\
    Return a 400 Bad Request response
    """
    response = Response(['Bad request'], status=400)


class Unauthorized(ResponseException):
    """\
    Return a 401 Unauthorized response.

    Use this when you want to flag that the user does not have permission to
    access a resource but may be able to gain access, eg by logging in.
    """
    response = Response(['Unauthorized'], status=401)


class Forbidden(ResponseException):
    """\
    Return a 403 Forbidden response

    Use this when you want to flag that the user does not have permission to
    access a resource and that there is no possiblity to do so (eg they are
    already logged in, but their account does not have the right access
    permissions)
    """
    response = Response(['Forbidden'], status=403)


class RedirectTemporary(ResponseException):
    """\
    Return a 302 Found response. Example::

        raise RedirectTemporary('http://example.org/')
    """
    def __init__(self, *args, **kwargs):
        self.response = Response.redirect(*args, **kwargs)

Redirect = RedirectTemporary


class RedirectPermanent(ResponseException):
    """\
    Return a 301 Moved Permanently response. Example::

        raise RedirectPermanent('http://example.org/')

    """
    def __init__(self, *args, **kwargs):
        self.response = Response.redirect_permanent(*args, **kwargs)


class RequestParseError(BadRequest):
    """\
    An error was encountered while parsing the HTTP request.
    """



from nose.tools import assert_equal

from fresco import FrescoApp

context = FrescoApp().requestcontext


class TestRequestProperties(object):

    def test_url_script_name_only(self):
        with context(SCRIPT_NAME='/foo/bar', PATH_INFO='') as c:
            assert_equal(c.request.url, "http://localhost/foo/bar")

    def test_url_script_name_path_info(self):
        with context(SCRIPT_NAME='/foo/bar', PATH_INFO='/baz') as c:
            assert_equal(c.request.url, "http://localhost/foo/bar/baz")

    def test_url_normalizes_host_port(self):
        with context(HTTP_HOST='localhost:80') as c:
            assert_equal(c.request.url, "http://localhost/")
        with context(HTTP_HOST='localhost:81') as c:
            assert_equal(c.request.url, "http://localhost:81/")

    def test_url_normalizes_host_ssl_port(self):
        with context(environ={'wsgi.url_scheme': 'https'},
                     HTTP_HOST='localhost:443') as c:
            assert_equal(c.request.url, "https://localhost/")

    def test_url_ignores_server_port_if_host_header_present(self):
        with context(environ={'wsgi.url_scheme': 'https', 'SERVER_PORT': '81'},
                     HTTP_HOST='localhost') as c:
            assert_equal(c.request.url, "https://localhost/")

    def test_as_string(self):
        with context('http://example.org/foo?bar=baz') as c:
            assert_equal(str(c.request),
                         '<Request GET http://example.org/foo?bar=baz>')

    def test_url_returns_full_url(self):
        with context('http://example.org/foo?bar=baz') as c:
            assert_equal(c.request.url, 'http://example.org/foo?bar=baz')

    def test_path_returns_correct_path_when_script_name_empty(self):
        with context(SCRIPT_NAME='', PATH_INFO='/foo/bar') as c:
            assert_equal(c.request.path, '/foo/bar')

    def test_path_returns_correct_path(self):
        with context(SCRIPT_NAME='/foo', PATH_INFO='/bar') as c:
            assert_equal(c.request.path, '/foo/bar')


class TestMakeURL(object):

    def test_returns_request_url(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.make_url(),
                         'http://localhost/script/pathinfo')

    def test_can_replace_path(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.make_url(path='/foo'),
                         'http://localhost/foo')

    def test_query_not_included_by_default(self):
        with context(QUERY_STRING='query=foo') as c:
            assert_equal(c.request.make_url(),
                         'http://localhost/')

    def test_query_dict(self):
        with context() as c:
            assert_equal(c.request.make_url(query={'a': 1, 'b': '2 3'}),
                         'http://localhost/?a=1;b=2+3')

    def test_query_kwargs(self):
        with context() as c:
            assert_equal(c.request.make_url(query={'a': 1}, b=2),
                         'http://localhost/?a=1;b=2')



class TestResolveURL(object):

    def test_relative_path(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('foo'),
                         'http://localhost/script/foo')

    def test_absolute_path_unspecified_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('/foo', relative='app'),
                         'http://localhost/script/foo')

    def test_absolute_path_app_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('/foo', relative='app'),
                         'http://localhost/script/foo')

    def test_absolute_path_server_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('/foo', relative='server'),
                         'http://localhost/foo')

    def test_ignores_server_port_if_host_header_present(self):
        with context(environ={'wsgi.url_scheme': 'https', 'SERVER_PORT': '81'},
                     HTTP_HOST='localhost') as c:
            assert_equal(c.request.resolve_url('/foo'),
                         'https://localhost/foo')


class TestCurrentRequest(object):

    def test_currentrequest_returns_current_request(self):
        from fresco import currentrequest
        with context() as c:
            assert currentrequest() is c.request

    def test_currentrequest_returns_None(self):
        from fresco import currentrequest
        assert currentrequest() is None

#def test_parsing_cookie_header():
#    @to_wsgi
#    def app(request):
#        return Response([
#            str(cookie) + '\n' for name, cookie in request.cookies.allitems()
#        ])
#
#    assert_equal(
#        TestApp(app).get(HTTP_COOKIE='$Version=1; NAME=a; $Path="/"; $Domain=".some.domain.org"; NAME=b;').body,
#        "NAME=a;Domain=.some.domain.org;Path=/;Version=1\n"
#        "NAME=b;Version=1\n",
#    )
#
#
#def test_method_not_allowed_dispatcher():
#    dispatcher = dispatcher_app()
#
#    @dispatcher.match('/blah', 'DELETE', 'PUT')
#    def app(request):
#        return Response.method_not_allowed(request)
#
#    response = TestApp(dispatcher).get('/blah')
#    assert 'not allowed' in response.body.lower()
#    assert_equal(response.status, "405 Method Not Allowed")
#    assert_equal(response.get_header('Allow'), 'PUT,DELETE')
#

#def test_request_path():
#
#    request = Request(make_environ(SCRIPT_NAME='/foo', PATH_INFO='/bar', HTTP_HOST='localhost:80'))
#    assert_equal(request.request_path, "/foo/bar")
#
#    request = Request(make_environ(SCRIPT_NAME='/foo/bar', PATH_INFO='', HTTP_HOST='localhost:80'))
#    assert_equal(request.request_path, "/foo/bar")
#
#    request = Request(make_environ(SCRIPT_NAME='/', PATH_INFO='foo/bar', HTTP_HOST='localhost:80'))
#    assert_equal(request.request_path, "/foo/bar")
#
#def test_form():
#
#    request = Request(make_environ(QUERY_STRING='foo=bar'))
#    assert_equal(request.get('foo'), u'bar')
#    assert_equal(request['foo'], u'bar')
#    assert_equal(request.get('fuz'), None)
#
#    request = Request(make_environ(QUERY_STRING='foo=bar&foo=baz'))
#    assert_equal(request.get('foo'), u'bar')
#    assert_equal(request['foo'], u'bar')
#    assert_equal(request.getlist('foo'), [u'bar', u'baz'])
#
#def test_remote_addr():
#    request = Request(make_environ(REMOTE_ADDR="1.2.3.4", HTTP_X_FORWARDED_FOR="4.3.2.1"))
#    assert request.remote_addr == "1.2.3.4"
#
#
#def test_encoding():
#
#    # Test UTF-8 decoding works (this should be the default)
#    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf8'))))
#    assert_equal(request.get('foo'), u'\xa0')
#
#    # Force a different encoding
#    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf7'))))
#    request.charset = 'utf-7'
#    assert_equal(request.get('foo'), u'\xa0')
#
#    # Force an incorrect encoding
#    request = Request(make_environ(QUERY_STRING=make_query(foo=u'\xa0'.encode('utf8'))))
#    request.charset = 'utf-7'
#    assert_raises(RequestParseError, request.get, 'foo')
#
#def test_form_getters():
#
#    alpha = u'\u03b1' # Greek alpha
#    beta = u'\u03b2' # Greek beta
#    gamma = u'\u03b3' # Greek gamma
#
#    request = Request(make_environ(
#        QUERY_STRING='foo=%s;foo=%s;bar=%s' % (
#            quote(alpha.encode('utf-8')),
#            quote(beta.encode('utf-8')),
#            quote(gamma.encode('utf-8'))
#        )
#    ))
#    assert request.get('foo') == request.form.get('foo') == alpha
#    assert request.getlist('foo') == request.form.getlist('foo') == [alpha, beta]
#
#    assert request.get('cheese') == request.form.get('cheese') == None
#    assert request.getlist('cheese') == request.form.getlist('cheese') == []
#
#    assert request.get('bar') == request.form.get('bar') == gamma
#    assert request.getlist('bar') == request.form.getlist('bar') == [gamma]
#
#
#def test_file_test():
#
#    filename = 'test.txt'
#    filedata = 'whoa nelly!\n'
#    content_type = 'text/plain'
#    boundary = '---------------------------1234'
#
#    post_data = (
#          '--%s\r\n' % boundary
#        + 'Content-Disposition: form-data; name="uploaded_file"; filename="%s"\r\n' % filename
#        + 'Content-Type: %s\r\n' % content_type
#        + '\r\n'
#        + filedata + '\r\n--' + boundary + '--\r\n'
#    )
#    request = Request(make_environ(
#        wsgi_input=StringIO(post_data),
#        REQUEST_METHOD='POST',
#        CONTENT_TYPE='multipart/form-data; boundary=%s' % boundary,
#        CONTENT_LENGTH=str(len(post_data)),
#    ))
#    assert_equal(request.files['uploaded_file'].filename, filename)
#    assert_equal(request.files['uploaded_file'].headers['Content-Type'], content_type)
#
#    ntf = NamedTemporaryFile()
#    request.files['uploaded_file'].save(ntf)
#    ntf.flush()
#
#    f = open(ntf.name, 'r')
#    testdata = f.read()
#    f.close()
#    ntf.close()
#    assert_equal(testdata, filedata)
#
#    ntf = NamedTemporaryFile()
#    request.files['uploaded_file'].save(ntf.name)
#
#    f = open(ntf.name, 'r')
#    testdata = f.read()
#    f.close()
#    ntf.close()
#    assert_equal(testdata, filedata)
#



from tempfile import NamedTemporaryFile
from fresco.options import Options


class TestOptions(object):

    def test_options_dictionary_access(self):
        options = Options()
        options['x'] = 1
        assert options['x'] == 1

    def test_options_attribute_access(self):
        options = Options()
        options.x = 1
        assert options.x == 1

    def test_options_update_from_object(self):

        class Foo:
            a = 1
            b = 2

        options = Options()
        options.update_from_object(Foo())
        assert options['a'] == 1
        assert options['b'] == 2

    def test_options_update_from_file(self):

        with NamedTemporaryFile() as tmpfile:
            tmpfile.write("a = 1\nb = 2\n")
            tmpfile.flush()

            options = Options()
            options.update_from_file(tmpfile.name)
            assert options['a'] == 1
            assert options['b'] == 2



import inspect
import re
from functools import partial
from itertools import cycle
from operator import attrgetter, itemgetter, methodcaller

from fresco.exceptions import BadRequest
from fresco.util.urls import join_path

__all__ = 'Pattern', 'Route', 'FormArg', 'QueryArg', 'CookieArg'


class URLGenerationError(Exception):
    """\
    Was not possible to generate the requested URL
    """


class Pattern(object):
    """\
    Patterns are matchable against URL paths using their ``match`` method. If a
    path matches, this should return a tuple of ``(positional_arguments,
    keyword_arguments)`` extracted from the URL path. Otherwise this method
    should return ``None``.

    Pattern objects may also be able to take a tuple of
    ``(positional_arguments, keyword_arguments)`` and return a corresponding
    URL path.
    """

    def match(self, path):
        """
        Should return a tuple of ``(positional_arguments, keyword_arguments)``
        if the pattern matches the given URL path, or None if it does not
        match.
        """
        raise NotImplementedError

    def pathfor(self, *args, **kwargs):
        """
        The inverse of ``match``: where possible, should generate a URL path
        for the given positional and keyword arguments.
        """
        raise NotImplementedError()

    def add_prefix(self, prefix):
        """
        Return a copy of the pattern with the given string prepended
        """
        raise NotImplementedError()


class Converter(object):
    """\
    Responsible for converting arguments to and from URL components.

    A ``Converter`` class should provide two instance methods:

    - ``to_string``: convert from a python object to a string
    - ``from_string``: convert from URL-encoded bytestring to the target
                        python type.

    It must also define the regular expression pattern that is used to extract
    the string from the URL.
    """
    pattern = '[^/]+'

    def __init__(self, pattern=None):
        """
        Initialize a ``Converter`` instance.
        """
        if pattern is not None:
            self.pattern = pattern

    def to_string(self, ob):
        """
        Convert arbitrary argument ``ob`` to a string representation
        """
        return unicode(ob)

    def from_string(self, s):
        """
        Convert string argument ``a`` to the target object representation,
        whatever that may be.
        """
        raise NotImplementedError


class IntConverter(Converter):
    """\
    Match any integer value and convert to an ``int`` value.
    """

    pattern = r'[+-]?\d+'

    def from_string(self, s):
        """
        Return ``s`` converted to an ``int`` value.
        """
        return int(s)


class UnicodeConverter(Converter):
    """\
    Match any string, not including a forward slash, and return a ``unicode``
    value
    """

    pattern = r'[^/]+'

    def to_string(self, s):
        """
        Return ``s`` converted to an ``unicode`` object.
        """
        return s

    def from_string(self, s):
        """
        Return ``s`` converted to an ``unicode`` object.
        """
        return unicode(s)


class AnyConverter(UnicodeConverter):
    """
    Match any one of the given string options.
    """

    pattern = r'[+-]?\d+'

    def __init__(self, *args):
        super(AnyConverter, self).__init__(None)
        if len(args) == 0:
            raise ValueError("Must supply at least one argument to any()")
        self.pattern = '|'.join(re.escape(arg) for arg in args)


class PathConverter(UnicodeConverter):
    """\
    Match any string, possibly including forward slashes, and return a
    ``unicode`` object.
    """
    pattern = r'.+'


class MatchAllURLsPattern(Pattern):
    """\
    A pattern matcher that matches all URLs starting with the given prefix. No
    arguments are parsed from the URL.
    """

    def __init__(self, path):
        self.path = path

    def match(self, path):
        if path.startswith(self.path):
            return (), {}
        return None

    def pathfor(self, *args, **kwargs):
        assert not args and not kwargs, \
                "MatchAllURLsPattern does not handle URL based arguments"

        return self.path

    def add_prefix(self, prefix):
        return self.__class__(join_path(prefix, self.path))

    def __unicode__(self):
        return '%s*' % (self.path,)


class ExtensiblePattern(Pattern):
    """\
    An extensible URL pattern matcher.

    Synopsis::

        >>> from pprint import pprint
        >>> p = ExtensiblePattern(r"/<:unicode>/<year:int>/<title:unicode>")
        >>> pprint(p.match('/archive/1999/blah'))
        ((u'archive',), {'title': u'blah', 'year': 1999})

    Patterns are split on slashes into components. A component can either be a
    literal part of the path, or a pattern component in the form::

        <identifier>:<converter>

    ``identifer`` can be any python name, which will be used as the name of a
    keyword argument to the matched function. If omitted, the argument will be
    passed as a positional arg.

    ``converter`` must be the name of a pre-registered converter. Converters
    must support ``to_string`` and ``from_string`` methods and are used to
    convert between URL segments and python objects.

    By default, the following converters are configured:

    - ``int`` - converts to an integer
    - ``path`` - any path (ie can include forward slashes)
    - ``unicode`` - any unicode string (not including forward slashes)
    - ``any`` - a string matching a list of alternatives

    Some examples::

        >>> p = ExtensiblePattern(r"/images/<:path>")
        >>> p.match('/images/thumbnails/02.jpg')
        ((u'thumbnails/02.jpg',), {})

        >>> p = ExtensiblePattern("/<page:any('about', 'help')>.html")
        >>> p.match('/about.html')
        ((), {'page': u'about'})

        >>> p = ExtensiblePattern("/entries/<id:int>")
        >>> p.match('/entries/23')
        ((), {'id': 23})

    Others can be added by calling ``ExtensiblePattern.register_converter``
    """

    preset_patterns = (
        ('int', IntConverter),
        ('unicode', UnicodeConverter),
        ('path', PathConverter),
        ('any', AnyConverter),
    )
    pattern_parser = re.compile("""
        <
            (?P<name>\w[\w\d]*)?
            :
            (?P<converter>\w[\w\d]*)
            (?:
                \(
                         (?P<args>.*?)
                \)
            )?
        >
    """, re.X)

    class Segment(object):
        """
        Represent a single segment of a URL, storing information about hte
        ``source``, ``regex`` used to pattern match the segment, ``name`` for
        named parameters and the ``converter`` used to map the value to a URL
        parameter if applicable
        """

        def __init__(self, source, regex, name, converter):
            self.source = source
            self.regex = regex
            self.name = name
            self.converter = converter

    def __init__(self, pattern, name=''):
        """
        Initialize a new ``ExtensiblePattern`` object with pattern ``pattern``
        and an optional name.
        """
        super(ExtensiblePattern, self).__init__()

        self.name = name
        self.preset_patterns = dict(self.preset_patterns)
        self.pattern = unicode(pattern)

        self.segments = list(self._make_segments(pattern))
        self.args = [item
                     for item in self.segments
                     if item.converter is not None]
        self.regex = re.compile(''.join(segment.regex
                                        for segment in self.segments) + '$')

    def _make_segments(self, s):
        r"""
        Generate successive Segment objects from the given string.

        Each segment object represents a part of the pattern to be matched, and
        comprises ``source``, ``regex``, ``name`` (if a named parameter) and
        ``converter`` (if a parameter)
        """

        for item in split_iter(self.pattern_parser, self.pattern):
            if isinstance(item, unicode):
                yield self.Segment(item, re.escape(item), None, None)
                continue
            groups = item.groupdict()
            name, converter, args = (groups['name'], groups['converter'],
                                     groups['args'])
            if isinstance(name, unicode):
                # Name must be a Python identifiers
                name = name.encode("ASCII")
            converter = self.preset_patterns[converter]
            if args:
                args, kwargs = self.parseargs(args)
                converter = converter(*args, **kwargs)
            else:
                converter = converter()
            yield self.Segment(item.group(0),
                               '(%s)' % converter.pattern, name, converter)

    def parseargs(self, argstr):
        """
        Return a tuple of ``(args, kwargs)`` parsed out of a string in the
        format ``arg1, arg2, param=arg3``.

        Synopsis::

            >>> ep =  ExtensiblePattern('')
            >>> ep.parseargs("1, 2, 'buckle my shoe'")
            ((1, 2, 'buckle my shoe'), {})
            >>> ep.parseargs("3, four='knock on the door'")
            ((3,), {'four': 'knock on the door'})

        """
        return eval('(lambda *args, **kwargs: (args, kwargs))(%s)' % argstr)

    def match(self, uri):
        """
        Test ``uri`` and return a tuple of parsed ``(args, kwargs)``, or
        ``None`` if there was no match.
        """
        mo = self.regex.match(uri)
        if not mo:
            return None
        groups = mo.groups()
        assert len(groups) == len(self.args), (
                "Number of regex groups does not match expected count. "
                "Perhaps you have used capturing parentheses somewhere? "
                "The pattern matched was %r." % self.regex.pattern
        )

        try:
            groups = [
                (segment.name, segment.converter.from_string(value))
                  for value, segment in zip(groups, self.args)
            ]
        except ValueError:
            return None

        args = tuple(value for name, value in groups if not name)
        kwargs = dict((name, value) for name, value in groups if name)
        return args, kwargs

    def pathfor(self, *args, **kwargs):
        """
        Example usage::

            >>> p = ExtensiblePattern("/view/<name:unicode>/<revision:int>")
            >>> p.pathfor(name='important_document.pdf', revision=299)
            u'/view/important_document.pdf/299'

            >>> p = ExtensiblePattern("/view/<:unicode>/<:int>")
            >>> p.pathfor('important_document.pdf', 299)
            u'/view/important_document.pdf/299'
        """

        args = list(args)
        result = []
        for seg in self.segments:
            if not seg.converter:
                result.append(seg.source)
            elif seg.name:
                try:
                    result.append(seg.converter.to_string(kwargs[seg.name]))
                except IndexError:
                    raise URLGenerationError(
                        "Argument %r not specified for url %r" % (
                            seg.name, self.pattern
                        )
                    )
            else:
                try:
                    result.append(seg.converter.to_string(args.pop(0)))
                except IndexError:
                    raise URLGenerationError(
                        "Not enough positional arguments for url %r" % (
                            self.pattern,
                        )
                    )
        return ''.join(result)

    def add_prefix(self, prefix):

        return self.__class__(
            join_path(prefix, self.pattern),
            self.name
        )

    @classmethod
    def register_converter(cls, name, converter):
        """
        Register a preset pattern for later use in URL patterns.

        Example usage::

            >>> from datetime import date
            >>> from time import strptime
            >>> class DateConverter(Converter):
            ...     pattern = r'\d{8}'
            ...     def from_string(self, s):
            ...         return date(*strptime(s, '%d%m%Y')[:3])
            ...
            >>> ExtensiblePattern.register_converter('date', DateConverter)
            >>> ExtensiblePattern('/<:date>').match('/01011970')
            ((datetime.date(1970, 1, 1),), {})
        """
        cls.preset_patterns += ((name, converter),)

    def __repr__(self):
        return '<%s %r>' % (self.__class__, self.pattern)

    def __unicode__(self):
        return u'%s' % (self.pattern,)


class Route(object):
    """\
    Represent a URL routing pattern
    """

    __slots__ = 'tag', 'pattern', 'methods', 'view', 'view_kwargs',\
                'predicate', 'decorators', 'original_view'

    #: The default class to use for URL pattern matching
    pattern_class = ExtensiblePattern

    def __init__(self, pattern, methods, view, view_kwargs=None,
                 tag='__default__', predicate=None, decorators=None, **kwargs):

        """
        :param pattern:     A string that can be compiled into a path pattern
        :param methods:     The list of HTTP methods the view is bound to
                            ('GET', 'POST', etc)
        :param view:        The view function.
        :param view_kwargs: A dictionary of default keyword arguments to pass
                            to the view callable
        :param tag:         A tag that can later be used to retrieve the route
                            for URL generation
        :param predicate:   A callable that is used to decide whether to match
                            this pattern. The callable must take a ``Request``
                            object as its only parameter and return a boolean.
        :param decorators:  Decorator functions to apply to the view callable
                            before invoking it
        :param kwargs:      Other keyword aruments passed through to the view
                            callable.
        """

        if not isinstance(pattern, Pattern):
            pattern = self.pattern_class(pattern)

        if isinstance(methods, basestring):
            methods = [methods]

        self.pattern = pattern
        self.methods = set(methods)
        self.view_kwargs = dict(view_kwargs or {}, **kwargs)
        for name, arg in self.view_kwargs.items():
            if isinstance(arg, RouteKwarg):
                arg.configure(self, name)
        self.tag = tag
        self.predicate = predicate
        self.decorators = decorators or []
        self.set_view(view)

    def __unicode__(self):
        return u'%s %s => %r\n' % (
            ' '.join(self.methods),
            unicode(self.pattern),
            self.original_view
        )

    def __repr__(self):
        return u'<%s %s %r %s>' % (
            self.__class__.__name__,
            ','.join(self.methods),
            unicode(self.pattern),
            self.original_view
        )

    def set_view(self, view):
        decorated_view = view
        for d in (self.decorators or []):
            decorated_view = d(decorated_view)

        self.original_view = view
        self.view = decorated_view

    def resolve_view(self, instance):
        """
        Resolve a view spec string to the actual callable.
        """
        if callable(self.original_view):
            return
        self.set_view(getattr(instance, self.original_view))

    def add_prefix(self, path):
        """
        Return a copy of the Route object with the given path prepended to the
        routing pattern.
        """
        return self.__class__(self.pattern.add_prefix(path), self.methods,
                              view=self.original_view,
                              view_kwargs=self.view_kwargs, tag=self.tag,
                              predicate=self.predicate,
                              decorators=self.decorators)

    def url(self, *args, **kwargs):
        """\
        Return a URL corresponding to the route

        :param _request: the current request object.
        :param _scheme: the URL scheme to use (eg 'https' or 'http').
        :param _netloc: the network location to use (eg 'localhost:8000').
        :param _script_name: the SCRIPT_NAME path component
        :param _query: any query parameters, as a dict or list of
                       ``(key, value)`` tuples.
        :param _fragment: a URL fragment to append.

        Any other arguments or keyword args are fed to the ``pathfor`` method
        of the pattern>
        """
        from fresco import context

        request = kwargs.pop('_request', None)
        scheme = kwargs.pop('_request', None)
        netloc = kwargs.pop('_netloc', None)
        query = kwargs.pop('_query', {})
        script_name = kwargs.pop('_script_name', None)
        fragment = kwargs.pop('_fragment', None)

        if request is None:
            request = context.request

        return request.make_url(scheme=scheme, netloc=netloc,
                                PATH_INFO=self.pattern.pathfor(*args,
                                                               **kwargs),
                                SCRIPT_NAME=script_name, parameters='',
                                query=query, fragment=fragment)


def split_iter(pattern, string):
    """
    Generate alternate strings and match objects for all occurances of
    ``pattern`` in ``string``.
    """
    matcher = pattern.finditer(string)
    match = None
    pos = 0
    for match in matcher:
        yield string[pos:match.start()]
        yield match
        pos = match.end()
    yield string[pos:]


def resolve_viewspec(viewspec, stack_depth):
    """
    Resolve the given view spec

    :param viewspec: View specification (eg ``'mypackage.views:my_view'``)
    :param stack_depth: How far back to look in the stack for the caller's
                        context, for resolving imports relative to the caller.
    """
    if not isinstance(viewspec, basestring):
        return viewspec

    try:
        module_spec, attr_spec = viewspec.split(':')
    except ValueError:
        raise ValueError("Expected string like 'package.module:view_func';"\
                         " got %r'" % viewspec)

    ob = __import__(module_spec)
    for mod in module_spec.split('.')[1:]:
        ob = getattr(ob, mod)

    for attr in attr_spec.split('.'):
        ob = getattr(ob, attr)
    return ob


class RouteKwarg(object):
    """
    RouteKwarg objects can be used as keyword arguments in a route definition.
    RouteKwargs can extract information from the request and make it available
    to the view.

    For example a RouteKwarg could be developed that reads information from the
    request cookie::

        Route('/', GET, affiliate=CookieArg())

    A naive implementation ``CookieArg`` could look like this::

        class CookieArg(RouteKwarg):

            def __call__(self, request):
                try:
                    return request.cookies[self.name].value
                except KeyError,
                    return None

    When the route is constructed the RouteKwarg's ``configure`` method will
    be called with the Route object and the keyword name.

    At every request, the RouteKwarg instance will be called and expected to
    supply the value for its argument.
    """

    route = None
    name = None

    def configure(self, route, name):
        self.route = route
        self.name = name

    def __call__(self, request):
        return None


class RequestArg(RouteKwarg):
    """
    Extract a view keyword argument from the request object.

    """
    query = attrgetter('query')
    form = attrgetter('form')
    cookies = attrgetter('cookies')

    #: Source for the request variable
    source = form

    #: Exceptions that signal the converter could not do its job due to invalid
    #: input
    converter_exceptions = (ValueError, TypeError)

    _marker = object()

    def __init__(self, converter=None, key=None,
                 default=_marker, exception=BadRequest):

        self.formkey = key
        self.default = default
        self.required = default is self._marker
        self.exception = exception
        self.is_list = isinstance(converter, list)
        if self.is_list:
            self.converter = lambda vs: [c(v)
                                         for c, v in zip(cycle(converter), vs)]
        else:
            self.converter = converter

    def configure(self, route, name):
        if self.formkey is None:
            self.formkey = name
        if self.is_list:
            self.getter = methodcaller('getlist', self.formkey)
        else:
            self.getter = itemgetter(self.formkey)

    def __call__(self, request):
        try:
            value = self.getter(self.source(request))
        except KeyError:
            if self.required:
                raise self.exception('No value provided for %s' %\
                                     (self.formkey,))
            return self.default

        try:
            if self.converter is not None:
                value = self.converter(value)
        except self.converter_exceptions:
            raise self.exception('%r is not a valid value for %s' %\
                                 (value, self.formkey,))

        return value


class CookieArg(RequestArg):
    """\
    Extract a view keyword argument from ``request.cookies``.

    Example::

        @app.route('/', GET, message=CookieArg(key='msg', default=None))
        def view(request, message):
            return Response([message])
    """
    source = RequestArg.cookies

    def getter(self, cookies):
        if self.is_list:
            return [c.value for c in cookies.getlist(self.formkey)]
        else:
            return cookies[self.formkey].value

    def configure(self, route, name):
        if self.formkey is None:
            self.formkey = name


class QueryArg(RequestArg):
    """\
    Extract a view keyword argument from ``request.query``.

    Example::

        @app.route('/add', GET, a=QueryArg(int), b=QueryArg(int))
        def view(request, a, b):
            return Response(['a + b = %d' % (a + b)])
    """
    source = RequestArg.query


class FormArg(RequestArg):
    """\
    Extract a view keyword argument from ``request.form``.

    Example::

        @app.route('/mul', POST, a=FormArg(int), b=FormArg(int))
        def view(request, a, b):
            return Response(['a * b = %d' % (a * b)])

    """
    source = RequestArg.form



from mock import Mock
from nose.tools import assert_raises
from flea import TestAgent
from fresco import FrescoApp, urlfor, GET, POST, Response
from fresco.exceptions import BadRequest
from fresco.routing import resolve_viewspec, Route, \
                           RouteKwarg, CookieArg, FormArg, QueryArg
from .fixtures import CBV


class TestMethodDispatch(object):

    def test_route_is_dispatched_to_correct_method(self):

        getview = Mock(return_value=Response())
        postview = Mock(return_value=Response())
        app = FrescoApp()
        app.route('/', GET, getview)
        app.route('/', POST, postview)

        TestAgent(app).get('/')
        assert getview.call_count == 1
        assert postview.call_count == 0

        TestAgent(app).post('/')
        assert getview.call_count == 1
        assert postview.call_count == 1


class TestResolveViewSpec(object):

    def test_relative_import(self):
        assert resolve_viewspec('fresco.tests.fixtures:CBV', 1) is CBV

    def test_attribute(self):
        assert resolve_viewspec('fresco.tests.fixtures:CBV.index_html', 1) \
                == CBV.index_html
        assert id(resolve_viewspec('fresco.tests.fixtures:CBV.index_html', 1))\
                == id(CBV.index_html)
        # Curiously the object identity test fails. Irritating, but I don't
        # think it matters.
        #assert resolve_viewspec('fresco.tests.fixtures:CBV.index_html', 1) \
        #       is CBV.index_html


class TestRouteDecorators(object):

    def exclaim(self, func):
        def exclaim(request, *args, **kwargs):
            response = func(request, *args, **kwargs)
            return response.replace(content=[''.join(response.content) + '!'])
        return exclaim

    def test_decorator_is_applied(self):

        views = CBV('test')

        app = FrescoApp()
        app.route('/decorated', GET, views.index_html,
                  decorators=[self.exclaim])
        app.route('/plain', GET, views.index_html)

        with app.requestcontext('/decorated') as c:
            req = c.request
            assert app.view(req).content == ['test!']

        with app.requestcontext('/plain') as c:
            assert app.view(c.request).content == ['test']

    def test_decorator_works_with_urlfor(self):

        views = CBV('test')
        app = FrescoApp()
        app.route('/decorated', GET, views.index_html,
                  decorators=[self.exclaim])
        with app.requestcontext():
            assert urlfor(views.index_html, _app=app) == \
                    'http://localhost/decorated'


class TestPredicates(object):

    def test_predicate_match(self):

        def v1(request):
            return Response(['x'])

        def v2(request):
            return Response(['y'])

        app = FrescoApp()
        app.route('/', GET, v1, predicate=lambda request: 'x' in request.query)
        app.route('/', GET, v2, predicate=lambda request: 'y' in request.query)

        t = TestAgent(app)
        assert t.get('/?x=1', check_status=False).body == 'x'
        assert t.get('/?y=1', check_status=False).body == 'y'
        assert t.get('/', check_status=False).response.status_code == 404


class TestRouteKwargs(object):

    def test_routekwarg_configured(self):

        A = RouteKwarg()
        route = Route('/', GET, lambda r: None, x=A)
        assert A.route is route
        assert A.name is 'x'

    def test_routekwarg_value_passed(self):

        view = Mock()
        routekwarg = Mock(spec=RouteKwarg)
        routekwarg.return_value = 'xyzzy'

        app = FrescoApp()
        app.route('/', GET, view, x=routekwarg)
        with app.requestcontext('/') as c:
            app.view(c.request)
            routekwarg.assert_called_with(c.request)
            view.assert_called_with(c.request, x='xyzzy')

    def test_queryarg_value_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=QueryArg())
        with app.requestcontext('/?x=foo') as c:
            app.view(c.request)
            view.assert_called_with(c.request, x='foo')

    def test_formarg_value_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=FormArg())
        with app.requestcontext('/?x=foo') as c:
            app.view(c.request)
            view.assert_called_with(c.request, x='foo')

    def test_cookiearg_value_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=CookieArg())
        with app.requestcontext('/', HTTP_COOKIE='x=foo') as c:
            app.view(c.request)
            view.assert_called_with(c.request, x='foo')

    def test_cookiearg_listvalue_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=CookieArg([unicode]))
        with app.requestcontext('/', HTTP_COOKIE='x=foo;x=bar') as c:
            app.view(c.request)
            view.assert_called_with(c.request, x=['foo', 'bar'])

    def test_requestarg_value_converted(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=FormArg(float))
        with app.requestcontext('/?x=0') as c:
            app.view(c.request)
            view.assert_called_with(c.request, x=0.0)

    def test_requestarg_default_value(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=FormArg(default='d'))
        with app.requestcontext('/') as c:
            app.view(c.request)
            view.assert_called_with(c.request, x='d')

    def test_requestarg_raises_exception(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=FormArg())
        with app.requestcontext('/') as c:
            assert_raises(BadRequest, app.view, c.request)



"""
Utility functions for HTTP message parsing

Copyright (c) 2009-2012 Oliver Cope. All rights reserved.
See LICENSE.txt for terms of redistribution and use.
"""

import re
from email.message import Message
from email.parser import Parser
from urllib import unquote_plus
from shutil import copyfileobj

import fresco
from fresco.exceptions import RequestParseError
from fresco.util.io import ExpandableOutput, SizeLimitedInput, PutbackInput, \
                           DelimitedInput

KB = 1024
MB = 1024 * KB

#: Data chunk size to read from the input stream (wsgi.input)
CHUNK_SIZE = 8 * KB


class TooBig(RequestParseError):
    """\
    Request body is too big
    """
    def __init__(self, *args, **kwargs):
        super(TooBig, self).__init__(*args, **kwargs)
        self.response = fresco.response.Response.request_entity_too_large()


class MissingContentLength(RequestParseError):
    """\
    No ``Content-Length`` header given
    """
    def __init__(self, *args, **kwargs):
        super(MissingContentLength, self).__init__(*args, **kwargs)
        self.response = fresco.response.Response.length_required()


def dequote(s):
    """\
    Return ``s`` with surrounding quotes removed. Example usage::

        >>> dequote('foo')
        'foo'
        >>> dequote('"foo"')
        'foo'
    """
    if len(s) > 1 and s[0] == '"' == s[-1]:
        return s[1:-1]
    return s


def parse_header(header):
    """\
    Given a header, return a tuple of
    ``(value, [(parameter_name, parameter_value)])``.

    Example usage::

        >>> parse_header("text/html; charset=UTF-8")
        ('text/html', {'charset': 'UTF-8'})
        >>> parse_header("multipart/form-data; boundary=-------7d91772e200be")
        ('multipart/form-data', {'boundary': '-------7d91772e200be'})
    """
    items = header.split(';')
    pairs = [
        (name, dequote(value))
             for name, value in (
                 item.lstrip().split('=', 1)
                    for item in items[1:]
            )
    ]
    return (items[0], dict(pairs))


def parse_querystring(data, charset=None, strict=False, keep_blank_values=True,
                      pairsplitter=re.compile('[;&]').split):
    """\
    Return ``(key, value)`` pairs from the given querystring::

        >>> list(parse_querystring('green%20eggs=ham;me=sam+i+am'))
        [(u'green eggs', u'ham'), (u'me', u'sam i am')]

    :param charset: Character encoding used to decode values. If not specified,
                    ``fresco.DEFAULT_CHARSET`` will be used.

    :param keep_blank_values: if True, keys without associated values will be
                              returned as empty strings. if False, no key,
                              value pair will be returned.

    :param strict: if ``True``, a ``ValueError`` will be raised on parsing
                   errors.
    """

    if charset is None:
        charset = fresco.DEFAULT_CHARSET

    for item in pairsplitter(data):
        if not item:
            continue
        try:
            key, value = item.split('=', 1)
        except ValueError:
            if strict:
                raise RequestParseError("bad query field: %r" % (item,))
            if not keep_blank_values:
                continue
            key, value = item, ''

        try:
            yield (unquote_plus(key).decode(charset),
                   unquote_plus(value).decode(charset))
        except UnicodeDecodeError:
            raise RequestParseError("Invalid character data: can't decode"\
                                    " as %r" % (charset,))


def parse_post(environ, fp, default_charset=None, max_size=16 * KB,
               max_multipart_size=2 * MB):
    """\
    Parse the contents of an HTTP POST request, which may be either
    application/x-www-form-urlencoded or multipart/form-data encoded.

    Returned items are either tuples of (name, value) for simple string values
    or (name, FileUpload) for uploaded files.

    :param max_multipart_size: Maximum size of total data for a multipart form
                               submission

    :param max_size: The maximum size of data allowed to be read into memory.
                     For a application/x-www-form-urlencoded submission, this
                     is the maximum size of the entire data. For a
                     multipart/form-data submission, this is the maximum size
                     of any individual field (except file uploads).
    """
    ct, ct_params = parse_header(
        environ.get('CONTENT_TYPE', 'application/x-www-form-urlencoded')
    )

    if default_charset is None:
        default_charset = fresco.DEFAULT_CHARSET
    charset = ct_params.get('charset', default_charset)

    try:
        content_length = int(environ['CONTENT_LENGTH'])
    except (TypeError, ValueError, KeyError):
        raise MissingContentLength()

    if ct == 'application/x-www-form-urlencoded':
        if content_length > max_size:
            raise TooBig("Content Length exceeds permitted size")
        return parse_querystring(SizeLimitedInput(fp, content_length).read(),
                                 charset)
    else:
        if content_length > max_multipart_size:
            raise TooBig("Content Length exceeds permitted size")
        try:
            boundary = ct_params['boundary']
        except KeyError:
            raise RequestParseError("No boundary given in multipart/form-data"\
                                    " content-type")
        return parse_multipart(SizeLimitedInput(fp, content_length), boundary,
                               charset, max_size)


class HTTPMessage(Message):
    """
    Represent HTTP request message headers
    """


def parse_multipart(fp, boundary, default_charset, max_size):
    """
    Parse data encoded as ``multipart/form-data``. Generate tuples of::

        (<field-name>, <data>)

    Where ``data`` will be a string in the case of a regular input field, or a
    ``FileUpload`` instance if a file was uploaded.

    :param fp: input stream from which to read data
    :param boundary: multipart boundary string, as specified by the
                     ``Content-Disposition`` header
    :param default_charset: character set to use for encoding, if not specified
                            by a content-type header. In practice web browsers
                            don't supply a content-type header so this needs to
                            contain a sensible value.
    :param max_size: Maximum size in bytes for any non file upload part
    """

    boundary_size = len(boundary)
    if not boundary.startswith('--'):
        raise RequestParseError("Malformed boundary string:"\
                                " must start with '--' (rfc 2046)")

    if boundary_size > 72:
        raise RequestParseError("Malformed boundary string:"\
                                " must be no more than 70 characters, not"\
                                " counting the two leading hyphens (rfc 2046)")

    assert boundary_size + 2 < CHUNK_SIZE, \
          "CHUNK_SIZE cannot be smaller than the boundary string"

    if fp.read(2) != '--':
        raise RequestParseError("Malformed POST data: expected two hypens")

    if fp.read(boundary_size) != boundary:
        raise RequestParseError("Malformed POST data: expected boundary")

    if fp.read(2) != '\r\n':
        raise RequestParseError("Malformed POST data: expected CRLF")

    fp = PutbackInput(fp)

    while True:
        headers, data = _read_multipart_field(fp, boundary)
        try:
            _, params = parse_header(headers['Content-Disposition'])
        except KeyError:
            raise RequestParseError("Missing Content-Disposition header")

        try:
            name = params['name']
        except KeyError:
            raise RequestParseError("Missing name parameter in"\
                                    " Content-Disposition header")

        is_file_upload = 'Content-Type' in headers and 'filename' in params

        if is_file_upload:
            stream = data._stream
            stream.seek(0)
            yield name, FileUpload(params['filename'], headers, stream)

        else:
            charset = parse_header(headers.get('Content-Type', ''))[1]\
                      .get('charset', default_charset)
            if data.tell() > max_size:
                raise TooBig("Data block exceeds maximum permitted size")
            try:
                data.seek(0)
                yield name, data.read().decode(charset)
            except UnicodeDecodeError:
                raise RequestParseError("Invalid character data: can't decode"\
                                        " as %r" % (charset,))

        chunk = fp.read(2)
        if chunk == '--':
            if fp.peek(3) != '\r\n':
                raise RequestParseError("Expected terminating CRLF"\
                                        " at end of stream")
            break

        if chunk != '\r\n':
            raise RequestParseError("Expected CRLF after boundary")


def _read_multipart_field(fp, boundary):
    """
    Read a single part from a multipart/form-data message and return a tuple of
    ``(headers, data)``. Stream ``fp`` must be positioned at the start of the
    header block for the field.

    Return a tuple of ('<headers>', '<data>')

    ``headers`` is an instance of ``email.message.Message``.

    ``data`` is an instance of ``ExpandableOutput``.

    Note that this currently cannot handle nested multipart sections.
    """
    data = ExpandableOutput()
    headers = Parser(_class=HTTPMessage).parse(
        DelimitedInput(fp, '\r\n\r\n'),
        headersonly=True
    )
    fp = DelimitedInput(fp, '\r\n--' + boundary)
    # XXX: handle base64 encoding etc
    for chunk in iter(lambda: fp.read(CHUNK_SIZE), ''):
        data.write(chunk)
    data.flush()

    # Fallen off the end of the input without having read a complete field?
    if not fp.delimiter_found:
        raise RequestParseError("Incomplete data (expected boundary)")

    return headers, data


class FileUpload(object):
    """\
    Represent a file uploaded in an HTTP form submission
    """

    def __init__(self, filename, headers, fileob):

        self.filename = filename
        self.headers = headers
        self.file = fileob

        # UNC/Windows path
        if self.filename[:2] == '\\\\' or self.filename[1:3] == ':\\':
            self.filename = self.filename[self.filename.rfind('\\') + 1:]

    def save(self, fileob):
        """
        Save the upload to the file object or path ``fileob``

        :param fileob: a file-like object open for writing, or the path to the
                       file to be written
        """
        if isinstance(fileob, basestring):
            with open(fileob, 'w') as f:
                return self.save(f)

        self.file.seek(0)
        copyfileobj(self.file, fileob)



# Copyright (c) 2009 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

from cStringIO import StringIO

from nose.tools import assert_equal
from flea import TestAgent

from fresco.util.http import parse_querystring, parse_post
from fresco import FrescoApp, Response, GET, POST

from . import form_data


class TestParseQueryString(object):

    def p(self, value):
        return list(parse_querystring(value))

    def test_empty(self):
        assert_equal(self.p(''), [])

    def test_simple_key_value(self):
        assert_equal(self.p('a=b'), [('a', 'b')])

    def test_key_with_space(self):
        assert_equal(self.p('a+b=c'), [('a b', 'c')])

    def test_value_with_space(self):
        assert_equal(self.p('a=b+c'), [('a', 'b c')])

    def test_double_equals(self):
        assert_equal(self.p('a==b'), [('a', '=b')])

    def test_escaped_chars(self):
        assert_equal(self.p('%20==c%3D'), [(' ', '=c=')])


class TestParseMultipart(object):

    def test_multipart(self):
        for data in form_data.multipart_samples:
            io = StringIO(data['data'])
            io.seek(0)
            environ = {
                'CONTENT_LENGTH': data['content_length'],
                'CONTENT_TYPE': data['content_type'],
            }
            parsed = sorted(list(parse_post(environ, io, 'UTF-8')))

            assert_equal(
                [name for name, value in parsed],
                ["empty-text-input", "file-upload", "text-input-ascii",
                "text-input-unicode"]
            )

            assert_equal(parsed[0], ("empty-text-input", ""))
            assert_equal(parsed[2], ("text-input-ascii", "abcdef"))
            assert_equal(parsed[3], ("text-input-unicode",
                                    "\xce\xb1\xce\xb2\xce\xb3\xce\xb4"\
                                    .decode("utf8")))

            fieldname, fileupload = parsed[1]
            assert_equal(fieldname, "file-upload")
            assert_equal(fileupload.filename, "test.data")
            assert_equal(fileupload.headers['content-type'],
                        "application/octet-stream")
            assert_equal(fileupload.file.read(), form_data.FILE_UPLOAD_DATA)

    def test_fileupload_too_big(self):
        """\
        Verify that multipart/form-data encoded POST data raises an exception
        if the total data size exceeds request.MAX_SIZE bytes
        """

        def view(request):
            request.MAX_MULTIPART_SIZE = 500
            request.get('f1')
            return Response(['ok'])
        app = FrescoApp()
        app.route('/', POST, view)

        r = TestAgent(app).post_multipart(
            files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)],
            check_status=False
        )
        assert_equal(r.response.status, "413 Request Entity Too Large")

        r = TestAgent(app).post_multipart(
            files=[('f1', 'filename.txt', 'text/plain', 'x' * 400)],
            data={'f2': 'x' * 101},
            check_status=False
        )
        assert_equal(r.response.status, "413 Request Entity Too Large")

    def test_fileupload_with_invalid_content_length(self):

        def view(request):
            request.MAX_MULTIPART_SIZE = 500
            request.get('f1')
            return Response(['ok'])
        app = FrescoApp()
        app.route('/', POST, view)

        r = TestAgent(app).post_multipart(
            files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)],
            CONTENT_LENGTH="499",
            check_status=False
        )
        assert_equal(r.response.status, "400 Bad Request")

    def test_multipart_field_too_big(self):
        """
        Verify that multipart/form-data encoded POST data raises an exception
        if it contains a single field exceeding request.MAX_SIZE bytes
        """
        def view(request):
            request.MAX_MULTIPART_SIZE = 500
            request.MAX_SIZE = 100
            request.get('f1')
            return Response(['ok'])
        app = FrescoApp()
        app.route('/', POST, view)

        r = TestAgent(app).post_multipart(data=[('f1', 'x' * 200)],
                                        check_status=False)
        assert_equal(r.response.status, "413 Request Entity Too Large")


class TestParseFormEncodedData(object):

    def test_formencoded_data_too_big(self):
        """
        Verify that application/x-www-form-urlencoded POST data raises an
        exception if it exceeds request.MAX_SIZE bytes
        """
        def view(request):
            request.MAX_SIZE = 100
            request.get('f1')
            return Response(['ok'])
        app = FrescoApp()
        app.route('/', POST, view)

        r = TestAgent(app).post(data=[('f1', 'x' * 200)], check_status=False)
        assert_equal(r.response.status, "413 Request Entity Too Large")

    def test_non_utf8_data_posted(self):

        def view(request):
            request.charset = 'latin1'
            x = request.form['char']
            return Response([x.encode('utf8')])
        app = FrescoApp()
        app.route('/', POST, view)

        r = TestAgent(app).post(data={'char': u'\u00a3'.encode('latin1')},
                                check_status=False)
        assert_equal(r.response.body.decode('utf8'), u"\u00a3")

    def test_non_utf8_data_getted(self):

        def view(request):
            request.charset = 'latin1'
            x = request.form['char']
            return Response([x.encode('utf8')])
        app = FrescoApp()
        app.route('/', GET, view)

        r = TestAgent(app).get(data={'char': u'\u00a3'.encode('latin1')},
                            check_status=False)
        assert_equal(r.response.body.decode('utf8'), u"\u00a3")



from flea import TestAgent
from nose.tools import assert_equal
from fresco import Response
from fresco import FrescoApp, GET, POST
from fresco import Route, routefor, urlfor, context
from .fixtures import CBV


class TestFrescoApp(object):

    def test_route_operates_as_a_decorator(self):

        app = FrescoApp()

        @app.route('/', GET)
        def view(req):
            return Response(['ok'])

        assert TestAgent(app).get('/').body == 'ok'

    def test_route_decorator_sets_url_property(self):
        app = FrescoApp()

        @app.route('/', GET)
        def view(req):
            return Response(['ok'])

        assert callable(view.url)

    def test_route_operates_as_a_function(self):

        def view(req):
            return Response(['ok'])

        app = FrescoApp()
        app.route('/', GET, view)
        assert TestAgent(app).get('/').body == 'ok'

    def test_route_returns_route_instance(self):
        def view(req):
            return Response(['ok'])

        app = FrescoApp()
        assert isinstance(app.route('/', GET, view), Route)

    def test_route_http_methods(self):

        def view(request):
            return Response([request.environ['REQUEST_METHOD']])

        app = FrescoApp()
        app.route('/get', GET, view)
        app.route('/post', POST, view)

        assert TestAgent(app).get('/get', check_status=False).response.status_code == 200
        assert TestAgent(app).post('/get', data={}, check_status=False).response.status_code == 405

        assert TestAgent(app).get('/post', check_status=False).response.status_code == 405
        assert TestAgent(app).post('/post', data={}, check_status=False).response.status_code == 200

    def test_HEAD_request_delegated_to_GET_view(self):

        app = FrescoApp()

        @app.route('/', GET)
        def view(req):
            return Response(['ok'], x_original_view='GET')

        assert TestAgent(app).get('/').body == 'ok'
        assert TestAgent(app).get('/').response.get_header('X-Original-View') == 'GET'
        ta = TestAgent(app)
        assert ta._request(ta.make_environ('HEAD', '/',)).response.get_header('X-Original-View') == 'GET'

    def test_NotFound_observed_when_raised_in_handler(self):

        def app1(req):
            from fresco.exceptions import NotFound
            if 'foo' in req.path_info:
                raise NotFound()
            return Response(['app1'])

        def app2(req):
            return Response(['app2'])

        app = FrescoApp()
        app.route_all('/', GET, app1)
        app.route_all('/', GET, app2)
        assert_equal(TestAgent(app).get('/bar').body, 'app1')
        assert_equal(TestAgent(app).get('/foo').body, 'app2')

    def test_NotFoundFinal_observed_when_raised_in_handler(self):

        def app1(req):
            from fresco.exceptions import NotFoundFinal
            if 'foo' in req.path_info:
                raise NotFoundFinal()
            return Response(['app1'])

        def app2(req):
            return Response(['app2'])

        app = FrescoApp()
        app.route_all('/', GET, app1)
        app.route_all('/', GET, app2)
        assert_equal(TestAgent(app).get('/bar').body, 'app1')
        assert_equal(TestAgent(app).get('/foo/', check_status=False).response.status_code, 404)

    def test_apps_called_in_correct_order(self):

        def view(req, value=''):
            return Response([value])

        app = FrescoApp()
        app.route_all('/f', GET, view, value='foo')
        app.route_all('/', GET, view, value='bar')
        assert_equal(TestAgent(app).get('/f/bar').body, 'foo')
        assert_equal(TestAgent(app).get('/b/bar').body, 'bar')

    def test_wsgi_app_handles_response_exceptions(self):

        from fresco.exceptions import NotFound

        def view(req):
            raise NotFound()

        app = FrescoApp()
        app.route('/', GET, view)
        assert_equal(TestAgent(app).get('/', check_status=False).response.status_code, 404)

    def test_route_wsgi_app(self):

        def wsgiapp(environ, start_response):

            start_response('200 OK', [('Content-Type', 'application/x-pachyderm')])
            return ['pretty pink elephants']

        app = FrescoApp()
        app.route_wsgi('/', wsgiapp)
        r = TestAgent(app).get('/')
        assert_equal(r.body, 'pretty pink elephants')
        assert_equal(r.response.get_header('Content-Type'), 'application/x-pachyderm')

class TestIncludeApp(object):

    def test_include_app(self):

        app = FrescoApp()
        @app.route('/', GET)
        def view(req):
            return Response(['ok'])

        app2 = FrescoApp()
        app2.include('/app1', app)

        assert TestAgent(app2).get('/', check_status=False).response.status_code == 404
        assert TestAgent(app2).get('/app1/', check_status=False).body == 'ok'

    def test_include_app_url_methods(self):

        app = FrescoApp()
        @app.route('/', GET)
        def view(req):
            return Response(view.url())

        app2 = FrescoApp()
        app2.include('/app1', app)

        assert_equal(TestAgent(app2).get('/app1/').body, 'http://127.0.0.1:8080/app1/')

class TestTrailingSlashes(object):

    """\
    The general principle is that if a GET or HEAD request is received for a
    URL without a trailing slash and no match is found, the app will look for a
    URL with a trailing slash, and redirect the client if such a route exists.
    """

    def test_no_trailing_slash(self):

        def foo(request):
            return Response(['foo'])

        app = FrescoApp()
        app.route('/foo/', GET, foo)

        r = TestAgent(app).get('/foo', follow=False)
        assert_equal(r.response.get_header('location'), 'http://127.0.0.1:8080/foo/')
        assert_equal(r.response.status_code, 301)

class TestViewCollection(object):


    def test_appdef(self):

        app = FrescoApp()
        app.include('/', CBV('bananas!'))
        assert_equal(TestAgent(app).get('/').body, 'bananas!')

    def test_appdef_url_generation(self):

        foo = CBV('foo!')
        bar = CBV('bar!')
        baz = CBV('baz!')

        app = FrescoApp(foo)
        app.include('/bar', bar)
        app.include('/baz', baz)

        with app.requestcontext():
            assert_equal(urlfor(foo.index_html), 'http://localhost/')
            assert_equal(urlfor(bar.index_html), 'http://localhost/bar/')
            assert_equal(urlfor(baz.index_html), 'http://localhost/baz/')

    def test_instance_available_in_context(self):

        s = []
        class MyCBV(CBV):

            def index_html(self, request):
                from fresco import context
                s.append(context.view_self)
                return Response([])

        instance = MyCBV('foo!')
        app = FrescoApp(instance)

        TestAgent(app).get('/')

        assert s[0] is instance


class TestURLGeneration(object):

    def test_urlfor_with_view_function(self):

        app = FrescoApp()
        @app.route('/foo', GET)
        def view(req):
            return Response(['ok'])

        with app.requestcontext():
            assert_equal(urlfor(view), 'http://localhost/foo')

    def test_urlfor_allows_script_name(self):

        app = FrescoApp()
        @app.route('/foo', GET)
        def view(req):
            return Response(['ok'])

        with app.requestcontext():
            assert_equal(urlfor(view, _script_name='/abc'), 'http://localhost/abc/foo')

    def test_urlfor_with_string(self):
        global myviewfunc
        myviewfunc = lambda req: Response(['ok'])
        app = FrescoApp()
        app.route('/myviewfunc', GET, myviewfunc)
        with app.requestcontext():
            assert_equal(urlfor('fresco.tests.test_core:myviewfunc'), 'http://localhost/myviewfunc')

    def test_urlfor_drops_query(self):
        myviewfunc = lambda req: Response([])
        app = FrescoApp()
        app.route('/', GET, myviewfunc)
        with app.requestcontext():
            assert_equal(urlfor(myviewfunc), 'http://localhost/')

    def test_urlfor_generates_first_route(self):

        myviewfunc = lambda req: Response([])
        app = FrescoApp()
        app.route('/1', GET, myviewfunc)
        app.route('/2', GET, myviewfunc)
        with app.requestcontext():
            assert_equal(urlfor(myviewfunc), 'http://localhost/1')

    def test_urlfor_with_class_based_view_spec(self):

        app = FrescoApp()
        app.include('/foo', CBV('x'))
        with app.requestcontext():
            assert_equal(urlfor('fresco.tests.fixtures:CBV.index_html'),
                         'http://localhost/foo/')

    def test_urlfor_with_class_based_view_unbound_method(self):

        app = FrescoApp()
        app.include('/foo', CBV('x'))
        with app.requestcontext():
            assert_equal(urlfor(CBV.index_html), 'http://localhost/foo/')

    def test_routefor_with_view_function(self):

        def view(req):
            return Response(['ok'])

        app = FrescoApp()
        route = app.route('/foo', GET, view)

        with app.requestcontext():
            assert_equal(routefor(view), route)

    def test_routefor_with_string(self):
        global myviewfunc
        myviewfunc = lambda req: Response(['ok'])
        app = FrescoApp()
        route = app.route('/myviewfunc', GET, myviewfunc)
        with app.requestcontext():
            assert_equal(routefor('fresco.tests.test_core:myviewfunc'), route)

    def test_routefor_generates_first_route(self):

        myviewfunc = lambda req: Response([])
        app = FrescoApp()
        r1 = app.route('/1', GET, myviewfunc)
        app.route('/2', GET, myviewfunc)
        with app.requestcontext():
            assert_equal(routefor(myviewfunc), r1)


class TestContextAttributes(object):

    def test_app_is_set(self):

        def check_app(request, expected):
            assert context.app is expected
            return Response([])

        app1 = FrescoApp()
        app2 = FrescoApp()

        app1.route('/', GET, check_app, {'expected': app1})
        app2.route('/', GET, check_app, {'expected': app2})

        TestAgent(app1).get('/')
        TestAgent(app2).get('/')


class TestAppRequestContext(object):

    def test_creates_isolated_context(self):

        app = FrescoApp()
        with app.requestcontext():
            context.request = 'foo'

            with app.requestcontext():
                context.request = 'bar'
                assert context.request == 'bar'

            assert context.request == 'foo'

    def test_parses_full_url(self):

        with FrescoApp().requestcontext('https://arthur@example.org:123/?x=y'):
            assert context.request.environ['HTTPS'] == 'on'
            assert context.request.environ['REMOTE_USER'] == 'arthur'
            assert context.request.environ['HTTP_HOST'] == 'example.org:123'
            assert context.request.environ['SCRIPT_NAME'] == ''
            assert context.request.environ['PATH_INFO'] == '/'
            assert context.request.environ['QUERY_STRING'] == 'x=y'


class TestResponseExceptions(object):

    def test_exception_is_converted_to_response(self):

        from fresco.exceptions import RedirectTemporary

        def redirector(request):
            raise RedirectTemporary('/foo')

        app = FrescoApp()
        app.route('/', GET, redirector)

        r = TestAgent(app).get('/', follow=False)
        assert r.response.status_code == 302




import sys

from collections import defaultdict
from contextlib import contextmanager
from functools import partial
from io import BytesIO
from urllib import unquote
from urlparse import urlparse
from types import MethodType

import blinker

from fresco import Request
from fresco.response import Response
from fresco.util.urls import normpath

from fresco.exceptions import ResponseException, NotFound
from fresco.requestcontext import RequestContext
from fresco.routing import ExtensiblePattern, MatchAllURLsPattern, \
                           Route, resolve_viewspec, RouteKwarg
from fresco.options import Options

HTTP_METHODS = GET, HEAD, POST, PUT, DELETE = \
               'GET', 'HEAD', 'POST', 'PUT', 'DELETE'

__all__ = HTTP_METHODS + ('FrescoApp', 'context', 'routefor', 'urlfor')

#: Context for the current request; allows apps to access the current request
#: context as a pseudo-global var
context = RequestContext()

#: Class to use to instantiate request objects
request_class = Request


class FrescoApp(object):
    """\
    Fresco application class.
    """

    #: The default class to use for URL pattern matching
    pattern_class = ExtensiblePattern

    #: A stdlib logger object, or None
    logger = None

    def __init__(self, views=None, path=None):

        super(FrescoApp, self).__init__()

        from fresco.middleware import FlashMiddleware, context_middleware

        #: List of URL routes
        self.__routes__ = []

        #: List mapping view functions to routes
        self.__routed_views__ = {}

        #: A list of (middleware, args, kwargs) tuples
        self._middleware = []

        #: Middleware layers applied after any middleware added through
        #: :meth:`add_middleware`.
        self.core_middleware = [FlashMiddleware,
                                partial(context_middleware, frescoapp=self)]

        #: The WSGI application. Replaced every time new middleware is added.
        self._wsgi_app = self._make_wsgi_app()

        #: An options dictionary, for arbitrary application variables or
        #: configuration
        self.options = Options()

        self.signal_ns = blinker.Namespace()

        #: Sent when a request matches a route, immediately before the view is
        #: invoked.
        #: Receivers will be passed: route, view, view_args, view_kwargs
        self.route_matched = self.signal_ns.signal('route_matched')

        #: Sent Sent after a view function is invoked, before the response
        #: object is output.
        #: Receivers will be passed: view, request and response
        self.view_finished = self.signal_ns.signal('view_finished')

        if views:
            if path is None:
                path = '/'
            self.include(path, views)

    def __call__(self, environ, start_response):
        """\
        Call the app as a WSGI application
        """
        return self._wsgi_app(environ, start_response)

    def __unicode__(self):
        """\
        Unicode representation of the application and its configured routes
        """
        s = u'<%s\n' % self.__class__.__name__
        for route in self.__routes__:
            s += u'    ' + unicode(route)
        s += '>'
        return  s

    def view(self, request):
        """\
        Implement the fresco view interface
        """
        context.app = self
        environ = request.environ
        environ['fresco.app'] = self
        method = environ['REQUEST_METHOD'].upper()
        path = unquote(environ.get('PATH_INFO', '').decode(request.charset,
                                                           'replace'))

        if not path:
            path = u'/'

        for view, route, args, kwargs in self.gettarget(request, path,
                                                        method):
            for k, v in kwargs.items():
                if isinstance(v, RouteKwarg):
                    v = v(request)
                kwargs[k] = v

            environ['wsgiorg.routing_args'] = (args, kwargs)
            if route:
                context.view_self = getattr(route.original_view, 'im_self',
                                            None)
            try:
                self.route_matched.send(self, route=route, view=view,
                                        request=request)
                response = view(request, *args, **kwargs)
                self.view_finished.send(self, view=view, response=response)
                return response
            except NotFound:
                continue
            except ResponseException as e:
                return e.response

        if path[-1] != '/':
            for _ in self.gettarget(request, path + '/', method):
                return Response.redirect_permanent(path + '/')

        return Response.not_found()

    def add_middleware(self, middleware, *args, **kwargs):
        """\
        Add a WSGI middleware layer
        """
        self._middleware.append((middleware, args, kwargs))
        self._wsgi_app = self._make_wsgi_app()

    def _raw_wsgi_app(self, environ, start_response):
        return self.view(request_class(environ))(environ, start_response)

    def _make_wsgi_app(self):
        app = self._raw_wsgi_app
        for middleware, args, kwargs in self._middleware:
            app = middleware(app, *args, **kwargs)
        for middleware in self.core_middleware:
            app = middleware(app)
        return app

    def _add_url_method(self, func, route):
        """\
        Add a method at ``func.url`` that returns a URL generated from
        ``pattern``s pathfor method.
        """
        if hasattr(func, 'url'):
            return
        try:
            func.url = partial(urlfor, func)
        except AttributeError:
            # Cannot set function attributes on bound or unbound methods.
            # See http://www.python.org/dev/peps/pep-0232/
            pass

    def _add_route(self, route):
        self.__routes__.append(route)
        routes = self.__routed_views__.setdefault(route.original_view, {})

        # Allow lookup by unbound method too
        if isinstance(route.original_view, MethodType):
            unbound_method = getattr(route.original_view.im_class,
                                    route.original_view.__name__)
            self.__routed_views__[unbound_method] = routes

        routes.setdefault(route.tag, route)

        if '__default__' not in routes:
            routes['__default__'] = route

        self._add_url_method(route.original_view, route)

    def route(self, pattern, methods, view=None, *args, **kwargs):
        """
        Match a URL pattern to a view function. Can be used as a function
        decorator, in which case the ``view`` parameter should not be passed.

        :param pattern: A string that can be compiled into a path pattern
        :param methods: A list of HTTP methods the view is bound to
        :param view:    The view function. If not specified a function
                        decorator will be returned.

        Other parameters are as for the :class:`Route` constructor.
        """

        # Catch the common error of not providing a valid method
        if not isinstance(methods, basestring):
            try:
                methods = iter(methods)
            except TypeError:
                raise TypeError("Method argument must be a string or iterable")

        if view is None:
            def route_decorator(func):
                self._add_route(Route(pattern, methods, func, *args, **kwargs))
                return func
            return route_decorator

        route = Route(pattern, methods, view, *args, **kwargs)
        self._add_route(route)
        return route

    def route_wsgi(self, path, wsgiapp, *args, **kwargs):

        def fresco_wsgi_view(request):
            fake_start_response = lambda status, headers, exc_info=None: None
            return Response.from_wsgi(wsgiapp,
                                      request.environ,
                                      fake_start_response)

        return self.route_all(path, HTTP_METHODS, fresco_wsgi_view,
                              *args, **kwargs)

    def route_all(self, path, *args, **kwargs):
        """\
        Expose a view for all URLs starting with ``path``.

        :param path: the path prefix at which the view will be routed
        """
        return self.route(MatchAllURLsPattern(path), *args, **kwargs)

    def include(self, path, views):
        """
        Include a view collection at the given path.

        The included view collection's url properties will be modified to
        generate the prefixed URLs.

        :param path:  Path at which to include the views
        :param views: Any collection of views (must expose a ``__routes___``
                      attribute)
        """
        routes = list(r.add_prefix(path) for r in views.__routes__)
        for r in routes:
            r.resolve_view(views)
            self._add_route(r)

    def gettarget(self, request, path, method):
        """
        Generate routes matching the request URI.

        For each function matched, yield a tuple of::

            (view, route, positional_args, keyword_args)

        Where positional and keyword arguments are parsed from the URL
        """
        path = normpath(path)
        valid_methods = set()

        is_head = (method == 'HEAD')
        head_views = []

        matching_routes = []
        found = False

        for route in self.__routes__:
            result = route.pattern.match(path)
            if result is None:
                continue

            if route and route.predicate and not route.predicate(request):
                continue

            view_args, view_kwargs = result
            if route.view_kwargs:
                view_kwargs = dict(route.view_kwargs, **view_kwargs)

            matching_routes.append((route, view_args, view_kwargs))

            if method not in route.methods:
                valid_methods.update(route.methods)
                if is_head and 'GET' in route.methods:
                    head_views.append((_make_head_view(route.view), route,
                                       view_args, view_kwargs))
                continue

            found = True
            if self.logger:
                self.logger.info("gettarget: %s %r => %r", request.method, path,
                                 route.original_view)

            yield route.view, route, view_args, view_kwargs

        if found or not valid_methods:
            return

        if head_views:
            for item in head_views:
                yield item
            return

        if valid_methods:
            yield self.view_method_not_found, None, (valid_methods,), {}
            return

    def view_method_not_found(self, request, valid_methods):
        """
        Return a ``405 Method Not Allowed`` response.

        Called when a view matched the pattern but no HTTP methods matched.
        """
        return Response.method_not_allowed(valid_methods)

    @contextmanager
    def requestcontext(self, url='http://localhost/', environ=None, **kwargs):
        """
        Return the global :class:`fresco.requestcontext.RequestContext`
        instance, populated with a new request object modelling default
        WSGI environ values.

        Synopsis::

            >>> app = FrescoApp()
            >>> with app.requestcontext('http://www.example.com/view') as c:
            ...     print c.request.url
            ...
            http://www.example.com/view

        """
        url = urlparse(url)
        netloc = url.netloc
        user = ''
        if '@' in netloc:
            user, netloc = netloc.split('@', 1)

        if ':' in user:
            user, _ = user.split(':')[0]

        env_overrides = environ or {}
        env_overrides.update(kwargs)

        environ = {
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'localhost',
            'SERVER_PORT': '80',
            'SERVER_PROTOCOL': 'HTTP/1.0',
            'HTTP_HOST': netloc or 'localhost',
            'SCRIPT_NAME': '',
            'PATH_INFO': url.path,
            'REMOTE_ADDR': '127.0.0.1',
            'wsgi.version': (1, 0),
            'wsgi.url_scheme': url.scheme or 'http',
            'wsgi.input': BytesIO(),
            'wsgi.errors': sys.stderr,
            'wsgi.multithread': True,
            'wsgi.multiprocess': True,
            'wsgi.run_once': False,
        }
        if url.scheme == 'https':
            environ['HTTPS'] = 'on'
            environ['SERVER_PORT'] = '443'

        if user:
            environ['REMOTE_USER'] = user

        if url.query:
            environ['QUERY_STRING'] = url.query

        environ.update(env_overrides)
        context.push(request=request_class(environ), app=self)
        yield context
        context.pop()


def collect_keys(items):
    """
    [(k1, v1), (k1, v2), (k2, v3)] -> [(k1, [v1, v2]), (k2, [v3])]
    """
    d = defaultdict(list)
    for key, value in items:
        d[key].append(value)
    return d.items()


def _make_head_view(view):
    """\
    Take a view that responds to a ``GET`` request and adapt it to one that
    will handle ``HEAD`` requests.
    """

    def head_view(*args, **kwargs):
        return view(*args, **kwargs).replace(content=[], content_length=0)
    return head_view


def routefor(viewspec, tag='__default__', _app=None):
    """\
    Return the Route corresponding to the given view function

    :param viewspec: a view callable or a string in the form
                     'package.module:viewfunction'
    :param tag: The URL tag for retrieving a named URL.
    :param _app: The app object. If None, the current app will be used.
    """
    app = _app or context.app
    viewspec = resolve_viewspec(viewspec, stack_depth=2)
    return app.__routed_views__[viewspec][tag]


def urlfor(viewspec, _tag='__default__', **kwargs):
    """\
    Return the url for the given view function

    :param viewspec: a view callable or a string in the form
                     'package.module:viewfunction'
    :param _tag: The URL tag for retrieving a named URL.
    :param _app: The app object. If None, the current app will be used.
    :param _request: the current request object.
    :param _scheme: the URL scheme to use (eg 'https' or 'http').
    :param _netloc: the network location to use (eg 'localhost:8000').
    :param _script_name: the SCRIPT_NAME path component
    :param _query: any query parameters, as a dict or list of
                    ``(key, value)`` tuples.
    :param _fragment: a URL fragment to append.
    """
    app = kwargs.get('_app') or context.app
    viewspec = resolve_viewspec(viewspec, stack_depth=2)
    return app.__routed_views__[viewspec][_tag].url(**kwargs)



from functools import partial
from urllib import quote_plus, unquote_plus

from fresco import context
from fresco import core

class FlashMiddleware(object):
    """
    Middleware to facilitate setting and displaying messages to the user across
    an http redirect.

    Usage::

        from fresco import flash

        def app1(request):
            flash.info("Operation completed")
            flash.warn("1 warning encountered")
            return Response.redirect('/app2.html')

        app1 = FlashMiddleware(app1)

        def app2(request):
            return Response("<p>Message was: %s</p>" % request.environ['fresco.flash_messages'])

    """

    INFO = 'INFO'
    WARN = 'WARN'
    ERROR = 'ERROR'

    __name__ = "FlashMiddleware"

    def __init__(self, app):
        self.app = app

    def _flash(self, messages, message, category=INFO):
        messages.append((category, message))

    def _start_response(self, environ, start_response, status, headers, exc_info=None):
        messages = environ['fresco.flash_messages']
        script_name = environ.get('SCRIPT_NAME', '') or '/'

        # Redirect response: set flash cookie so the messages are persisted to the next request
        if status[:2] == '30' and messages:
            cookies = [(
                'Set-Cookie',
                'fresco.flash=%s;path=%s' % (
                    ','.join(('%s:%s' % (c, m)).encode("base64") for c, m in messages).strip().replace('\n',''),
                    script_name,
                )
            )]
        elif 'fresco.flash' in environ.get('HTTP_COOKIE', ''):
            cookies = [('Set-Cookie', 'fresco.flash=;Expires=Tue, 01 Jan 1980 00:00:00;Path=' + script_name)]
        else:
            cookies = []

        headers.extend(cookies)
        return start_response(status, headers, exc_info)

    def __call__(self, environ, start_response):
        request = core.request_class(environ)
        flash_in = request.cookies.get('fresco.flash', None)
        if flash_in and flash_in.value:
            messages = [item.decode('base64').split(':', 1) for item in flash_in.value.split(',')]
        else:
            messages = []
        environ['fresco.flash_messages'] = messages
        environ['fresco.flash'] = partial(self._flash, messages)

        return self.app(environ, partial(self._start_response, environ, start_response))

def info(message):
    context.request.environ['fresco.flash'](message, FlashMiddleware.INFO)

def warn(message):
    context.request.environ['fresco.flash'](message, FlashMiddleware.WARN)

def error(message):
    context.request.environ['fresco.flash'](message, FlashMiddleware.ERROR)

def messages():
    """
    Return a list of (level, message) tuples flashed from the previous request
    """
    return context.request.environ['fresco.flash_messages']




from flea import TestAgent

from fresco import flash, FrescoApp, GET, Response

app = FrescoApp()
@app.route('/messages', GET)
def flash_messages(request):
    return Response('%'.join(s for level, s in flash.messages()))

@app.route('/flash', GET)
def flash_custom(request):
    for item in request.query.getlist('m'):
        flash.info(item)
    if 'r' in request.query:
        return Response.redirect(
            request.query.get('r')
        )
    return Response('%'.join(s for level, s in flash.messages()))

app = TestAgent(app)

class TestFlash(object):

    def test_flash_with_redirect(self):
        r = app.get('/flash', data=[('r', '/messages'), ('m', 'hello')])
        assert r.request.request_path == '/messages'
        assert r.body == 'hello'

    def test_multi_flash_with_redirect(self):
        r = app.get('/flash', data=[('r', '/messages'), ('m', 'hello'), ('m', 'world')])
        assert r.request.request_path == '/messages'
        assert r.body == 'hello%world'

    def test_multi_flash_with_metachars_and_redirect(self):
        r = app.get('/flash', data=[('r', '/messages'), ('m', 'hel:lo'), ('m', 'wo,r;ld')])
        assert r.request.request_path == '/messages'
        assert r.body == 'hel:lo%wo,r;ld', r.body

    def test_flash(self):
        r = app.get('/flash', data=[('m', 'hello')])
        assert r.request.request_path == '/flash'
        assert r.body == 'hello'

    def test_multi_flash(self):
        r = app.get('/flash', data=[('m', 'hello'), ('m', 'world')])
        assert r.request.request_path == '/flash'
        assert r.body == 'hello%world'

    def test_multi_flash_with_metachars(self):
        r = app.get('/flash', data=[('m', 'hel:lo'), ('m', 'wo,r;ld')])
        assert r.request.request_path == '/flash'
        assert r.body == 'hel:lo%wo,r;ld'


